@@ -1,12 +1,14 @@
 """
 Plotting: Graphical representations of data extracted from datasets.
 
-Graphical representations of TREPR data are an indispensable aspect of data
+Graphical representations of tr-EPR data are an indispensable aspect of data
 analysis. To facilitate this, a series of different plotters are available.
 Additionally, savers (and stylers) are implemented.
 
 Plotting relies on `matplotlib <https://matplotlib.org/>`_, and mainly its
-object-oriented interface should be used for the actual plotting.
+object-oriented interface should be used for the actual plotting inside the
+actual plotter classes. Note that the user of the trepr package usually will
+not be exposed directly to the matplotlib interface.
 
 Generally, two types of plotters can be distinguished:
 
@@ -40,9 +42,10 @@ Something often quite confusing is the apparent inconsistency between the
 order of array dimensions and the order of axes. While we are used to assign
 axes in the order *x*, *y*, *z*, and assuming *x* to be horizontal,
 *y* vertical (and *z* sticking out of the paper plane), arrays are usually
-indexed row-first, column-second. That means, however, that if you simply
-plot a 2D array in axes, your *first* dimension is along the *y* axis,
-the *second* dimension along the *x* axis.
+indexed row-first, column-second (at least in the widely followed C
+convention; with FORTRAN things are different). That means, however, that if
+you simply plot a 2D array in axes, your *first* dimension is along the *y*
+axis, the *second* dimension along the *x* axis.
 
 Therefore, as the axes of your datasets will always correspond to the array
 dimensions of your data, in case of 2D plots you will need to *either* use
@@ -109,229 +112,32 @@ Concrete plotters for multiple datasets
   line-type plots, including (semi)log plots
 
 
+A note for developers
+=====================
+
+As each kind of spectroscopy comes with own needs for extensions, there is a
+class :class:`PlotterExtensions` that can be used as a mixin class for other
+plotters to provide additional functionality for all plotters.
+
+Make sure when implementing functionality here that it really works with all
+types of plotters, *i.e.* both SinglePlotters and MultiPlotters. This is
+particularly relevant if you need to get information from dataset(s),
+as a SinglePlotter will have an attribute ``dataset``, while a MultiPlotter
+will have an attribute ``datasets``.
+
+
 Module documentation
 ====================
 
 """
 
-from colour import Color
 import matplotlib as mpl
-import matplotlib.pyplot as plt
-from matplotlib import cm
 from matplotlib.colors import ListedColormap
 import numpy as np
 
 import aspecd.plotting
 import trepr.dataset
-
-left = Color(rgb=(102 / 255, 0 / 255, 204 / 255))
-middle = Color(rgb=(1, 1, 1))
-right = Color(rgb=(255 / 255, 51 / 255, 51 / 255))
-
-R_left = np.linspace(102 / 255, 1, 128)
-G_left = np.linspace(0, 1, 128)
-B_left = np.linspace(204 / 255, 1, 128)
-
-A_left = np.linspace(1, 0, 128)
-A_right = np.linspace(0, 1, 128)
-
-R_right = np.linspace(1, 255 / 255, 128)
-G_right = np.linspace(1, 51 / 255, 128)
-B_right = np.linspace(1, 51 / 255, 128)
-
-colors_left = list(left.range_to(middle, 128))
-colors_right = list(middle.range_to(right, 128))
-for i, _ in enumerate(colors_left):
-    colors_left[i] = list(colors_left[i].rgb)
-for i, _ in enumerate(colors_right):
-    colors_right[i] = list(colors_right[i].rgb)
-
-for i, _ in enumerate(colors_left):
-    colors_left[i][0] = R_left[i]
-    colors_left[i][1] = G_left[i]
-    colors_left[i][2] = B_left[i]
-
-for i, _ in enumerate(colors_right):
-    colors_right[i][0] = R_right[i]
-    colors_right[i][1] = G_right[i]
-    colors_right[i][2] = B_right[i]
-
-viridis = cm.get_cmap('viridis', 256)
-jara_colors = viridis(np.linspace(0, 1, 256))
-
-for i in range(0, 128):
-    jara_colors[i][0:3] = [102 / 255, 0, 204 / 255]
-    jara_colors[i][3] = A_left[i]
-for i in range(128, 255):
-    jara_colors[i][0:3] = [1, 51 / 255, 51 / 255]
-    jara_colors[i][3] = A_right[i - 128]
-
-jara_cmap = ListedColormap(jara_colors)
-
-
-class ScaledImagePlot(aspecd.plotting.SinglePlotter):
-    """Create a scaled image of a given dataset.
-
-    Attributes
-    ----------
-    style : str
-        Defines whether the plot is done in xkcd style or not.
-
-    description : str
-        Describes the aim of the class.
-
-
-    .. deprecated:: 0.1
-        Use :class:`SinglePlotter2D` instead.
-
-    """
-
-    def __init__(self):
-        super().__init__()
-        # public properties
-        self.style = ''
-        self.description = '2D plot as scaled image.'
-        # protected properties
-        self._extent = list()
-        self._style_dict = {'interpolation': 'bilinear',
-                            'cmap': viridis,
-                            'origin': 'lower',
-                            'aspect': 'auto'}
-        self._ticklabel_format = {'style': 'sci',
-                                  'scilimits': (0, 0),
-                                  'useMathText': True}
-        self.parameters['show_zero_lines'] = False
-
-    def _create_plot(self):
-        """Plot the given dataset with axes labels and a normed colormap."""
-        self._get_extent()
-        self._set_style()
-        self._display_data()
-        self._format_ticklabels()
-
-    def _set_style(self):
-        """Set the style to xkcd if indicated."""
-        if self.style == 'xkcd':
-            plt.xkcd()
-
-    def _display_data(self):
-        """Display the data with adjusted colormap."""
-        colormap_adjuster = ColormapAdjuster(dataset=self.dataset)
-        colormap_adjuster.adjust()
-        self.axes.imshow(self.dataset.data.data.T,
-                         norm=colormap_adjuster.normalised_colormap,
-                         extent=self._extent,
-                         **self._style_dict)
-
-    def _format_ticklabels(self):
-        """Set the format for the ticklabels."""
-        plt.ticklabel_format(axis='x', **self._ticklabel_format)
-
-    def _get_extent(self):
-        """Define the start and end values of the axes."""
-        x_axis_start = self.dataset.data.axes[0].values[0]
-        x_axis_end = self.dataset.data.axes[0].values[-1]
-        y_axis_start = self.dataset.data.axes[1].values[0]
-        y_axis_end = self.dataset.data.axes[1].values[-1]
-        self._extent = [x_axis_start, x_axis_end, y_axis_start, y_axis_end]
-
-
-class LinePlot(aspecd.plotting.SinglePlotter):
-    """Create a 1D line plot of a given dataset.
-
-    Attributes
-    ----------
-    style : str
-        Defines whether the plot is done in xkcd style or not.
-
-    description : str
-        Describes the aim of the class.
-
-
-    .. deprecated:: 0.1
-        Use :class:`SinglePlotter1D` instead.
-
-    """
-
-    def __init__(self):
-        super().__init__()
-        # public properties
-        self.style = ''
-        self.description = '1D line plot.'
-        self.parameters['color'] = None
-        # protected properties
-        self._zero_line_style = {'y': 0,
-                                 'color': '#999999'}
-        self._ticklabel_format = {'style': 'sci',
-                                  'scilimits': (-2, 4),
-                                  'useMathText': True}
-
-    def _create_plot(self):
-        """Plot the given dataset with axes labels and a zero line."""
-        self._set_style()
-        self._display_zero_line()
-        self._display_data()
-        self._set_axes()
-
-    def _set_style(self):
-        """Set the style to xkcd if indicated."""
-        if self.style == 'xkcd':
-            plt.xkcd()
-
-    def _display_zero_line(self):
-        """Create a horizontal line at zero."""
-        plt.axhline(**self._zero_line_style)
-
-    def _display_data(self):
-        """Plot the data."""
-        self.axes.plot(self.dataset.data.axes[0].values,
-                       self.dataset.data.data, color=self.parameters['color'])
-
-    def _set_axes(self):
-        """Set the limits of the x-axis as well as the ticklabel format."""
-        self.axes.set_xlim([self.dataset.data.axes[0].values[0],
-                            self.dataset.data.axes[0].values[-1]])
-        plt.ticklabel_format(**self._ticklabel_format)
-
-
-class MultiLinePlot(aspecd.plotting.MultiPlotter):
-    """Plot multiple datasets as lines.
-
-    .. deprecated:: 0.1
-        Use :class:`MultiPlotter1D` instead.
-    """
-
-    def __init__(self):
-        super().__init__()
-        self.description = '1D line plot for multiple lines.'
-        self.parameters['color'] = None
-        self._zero_line_style = {'y': 0,
-                                 'color': '#999999'}
-        self._ticklabel_format = {'style': 'sci',
-                                  'scilimits': (-2, 4),
-                                  'useMathText': True}
-
-    def _create_plot(self):
-        self._display_zero_line()
-        self._display_data()
-        self._set_axes()
-
-    def _display_zero_line(self):
-        """Create a horizontal line at zero."""
-        plt.axhline(**self._zero_line_style)
-
-    def _display_data(self):
-        for idx, dataset in enumerate(self.datasets):
-            if self.parameters['color']:
-                self.axes.plot(dataset.data.axes[0].values, dataset.data.data,
-                               color=self.parameters['color'][idx])
-            else:
-                self.axes.plot(dataset.data.axes[0].values, dataset.data.data)
-
-    def _set_axes(self):
-        self.axes.set_xlim([self.datasets[0].data.axes[0].values[0],
-                            self.datasets[0].data.axes[0].values[-1]])
-        plt.ticklabel_format(**self._ticklabel_format)
+from trepr import utils
 
 
 class ColormapAdjuster:
@@ -383,7 +189,65 @@ class ColormapAdjuster:
             mpl.colors.Normalize(vmin=self._bound * -1, vmax=self._bound)
 
 
-class SinglePlotter1D(aspecd.plotting.SinglePlotter1D):
+class PlotterExtensions:
+    # noinspection PyUnresolvedReferences
+    """Extensions for plots of tr-EPR data.
+
+    This class is meant as a mixin class for plotters of the trepr package
+    and provides functionality specific for tr-EPR-spectroscopic data.
+
+    Hence it can only be used as mixin in addition to a plotter class.
+
+    Attributes
+    ----------
+    parameters : :class:`dict`
+        All parameters necessary for the plot, implicit and explicit
+
+        The following keys exist, in addition to those defined by the actual
+        plotter:
+
+        g-axis: :class:`bool`
+            Whether to show an additional *g* axis opposite of the magnetic
+            field axis
+
+            This assumes the magnetic field axis to be the *x* axis and the
+            magnetic field values to be in millitesla (mT), as it calls
+            :func:`trepr.utils.convert_mT2g`.
+
+
+    .. versionadded:: 0.2
+
+    """
+
+    def __init__(self):
+        self.parameters['g-axis'] = False  # noqa
+
+    def _create_g_axis(self, mw_freq=None):
+        """
+        Add a *g* axis as second axis opposite the magnetic field axis.
+
+        Currently, this function assumes the magnetic field axis to be the
+        *x* axis. Additionally, the magnetic field values are assumed to be
+        in millitesla (mT), and the microwave frequency to be in gigahertz (
+        GHz).
+
+        Parameters
+        ----------
+        mw_freq : :class:`float`
+            microwave frequency (**in GHz**) used to convert from mT to g
+
+        """
+        def forward(values):
+            return utils.convert_mT2g(values, mw_freq=mw_freq)
+
+        def backward(values):
+            return utils.convert_g2mT(values, mw_freq=mw_freq)
+
+        gaxis = self.ax.secondary_xaxis('top', functions=(backward, forward))
+        gaxis.set_xlabel(r'$g\ value$')
+
+
+class SinglePlotter1D(aspecd.plotting.SinglePlotter1D, PlotterExtensions):
     """1D plots of single datasets.
 
     Convenience class taking care of 1D plots of single datasets.
@@ -392,6 +256,10 @@ class SinglePlotter1D(aspecd.plotting.SinglePlotter1D):
     ASpecD documentation of the :class:`aspecd.plotting.SinglePlotter1D`
     class for details.
 
+    Furthermore, the class inherits all functionality from
+    :class:`PlotterExtensions`. See there for additional details.
+
+
     Examples
     --------
     For convenience, a series of examples in recipe style (for details of
@@ -409,10 +277,29 @@ class SinglePlotter1D(aspecd.plotting.SinglePlotter1D):
          properties:
            filename: output.pdf
 
+
+    In case you would like to have a *g* axis plotted as a second *x* axis on
+    top (note that this only makes sense in case of a calibrated magnetic
+    field axis):
+
+    .. code-block:: yaml
+
+       - kind: singleplot
+         type: SinglePlotter1D
+         properties:
+           parameters:
+             g-axis: true
+           filename: output.pdf
+
     """
 
+    def _create_plot(self):
+        super()._create_plot()
+        if self.parameters['g-axis'] and self.dataset.data.axes[0].unit == 'mT':
+            self._create_g_axis(self.dataset.metadata.bridge.mw_frequency.value)
 
-class SinglePlotter2D(aspecd.plotting.SinglePlotter2D):
+
+class SinglePlotter2D(aspecd.plotting.SinglePlotter2D, PlotterExtensions):
     """2D plots of single datasets.
 
     Convenience class taking care of 2D plots of single datasets.
@@ -421,6 +308,10 @@ class SinglePlotter2D(aspecd.plotting.SinglePlotter2D):
     ASpecD documentation of the :class:`aspecd.plotting.SinglePlotter2D`
     class for details.
 
+    Furthermore, the class inhertis all functionality from
+    :class:`PlotterExtensions`. See there for additional details.
+
+
     Examples
     --------
     For convenience, a series of examples in recipe style (for details of
@@ -486,17 +377,125 @@ class SinglePlotter2D(aspecd.plotting.SinglePlotter2D):
     Make sure to check the documentation of the ASpecD
     :mod:`aspecd.plotting` module for further parameters that can be set.
 
+    In case you would like to have a *g* axis plotted as a second *x* axis on
+    top (note that this only makes sense in case of a calibrated magnetic
+    field axis):
+
+    .. code-block:: yaml
+
+       - kind: singleplot
+         type: SinglePlotter2D
+         properties:
+           parameters:
+             g-axis: true
+           filename: output.pdf
+
     """
 
+    def _create_plot(self):
+        super()._create_plot()
+        if self.parameters['g-axis'] and self.dataset.data.axes[0].unit == 'mT':
+            self._create_g_axis(self.dataset.metadata.bridge.mw_frequency.value)
 
-class MultiPlotter1D(aspecd.plotting.MultiPlotter1D):
+
+class SinglePlotter2DStacked(aspecd.plotting.SinglePlotter2DStacked,
+                             PlotterExtensions):
+    """Stacked plots of 2D data.
+
+    A stackplot creates a series of lines stacked on top of each other from
+    a 2D dataset.
+
+    As the class is fully inherited from ASpecD for simple usage, see the
+    ASpecD documentation of the :class:`aspecd.plotting.SinglePlotter2DStacked`
+    class for details.
+
+    Furthermore, the class inherits all functionality from
+    :class:`PlotterExtensions`. See there for additional details.
+
+
+    Examples
+    --------
+    For convenience, a series of examples in recipe style (for details of
+    the recipe-driven data analysis, see :mod:`aspecd.tasks`) is given below
+    for how to make use of this class. Of course, all parameters settable
+    for the superclasses can be set as well. The examples focus each on a
+    single aspect.
+
+    In the simplest case, just invoke the plotter with default values:
+
+    .. code-block:: yaml
+
+       - kind: singleplot
+         type: SinglePlotter2DStacked
+         properties:
+           filename: output.pdf
+
+    If you need to more precisely control the formatting of the y tick
+    labels, particularly the number of decimals shown, you can set the
+    formatting accordingly:
+
+    .. code-block:: yaml
+
+       - kind: singleplot
+         type: SinglePlotter2DStacked
+         properties:
+           filename: output.pdf
+           parameters:
+             yticklabelformat: '%.2f'
+
+    In this particular case, the y tick labels will appear with only two
+    decimals. Note that currently, the "old style" formatting specifications
+    are used due to their widespread use in other programming languages and
+    hence the familiarity of many users with this particular notation.
+
+    Sometimes you want to have horizontal "zero lines" appear for each
+    individual trace of the stacked plot. This can be achieved explicitly
+    setting the "show_zero_lines" parameter to "True" that is set to "False"
+    by default:
+
+    .. code-block:: yaml
+
+       - kind: singleplot
+         type: SinglePlotter2DStacked
+         properties:
+           filename: output.pdf
+           parameters:
+             show_zero_lines: True
+
+    In case you would like to have a *g* axis plotted as a second *x* axis on
+    top (note that this only makes sense in case of a calibrated magnetic
+    field axis):
+
+    .. code-block:: yaml
+
+       - kind: singleplot
+         type: SinglePlotter2DStacked
+         properties:
+           parameters:
+             g-axis: true
+           filename: output.pdf
+
+    """
+
+    def _create_plot(self):
+        super()._create_plot()
+        if self.parameters['g-axis'] and self.dataset.data.axes[0].unit == 'mT':
+            self._create_g_axis(self.dataset.metadata.bridge.mw_frequency.value)
+
+
+class MultiPlotter1D(aspecd.plotting.MultiPlotter1D, PlotterExtensions):
+    # noinspection PyUnresolvedReferences
     """1D plots of multiple datasets.
 
     Convenience class taking care of 1D plots of multiple datasets.
 
-    As the class is fully inherited from ASpecD for simple usage, see the
+    As the class is inherited from ASpecD for simple usage, see the
     ASpecD documentation of the :class:`aspecd.plotting.MultiPlotter1D`
-    class for details.
+    class for its general use.
+
+    Furthermore, the class inherits all functionality from
+    :class:`PlotterExtensions`. See there for additional details.
+
 
     Examples
     --------
@@ -539,10 +538,32 @@ class MultiPlotter1D(aspecd.plotting.MultiPlotter1D):
         ``#``, you need to explicitly tell YAML that these are strings,
         surrounding the values by quotation marks.
 
+    In case you would like to have a *g* axis plotted as a second *x* axis on
+    top (note that this only makes sense in case of a calibrated magnetic
+    field axis):
+
+    .. code-block:: yaml
+
+       - kind: multiplot
+         type: MultiPlotter1D
+         properties:
+           parameters:
+             g-axis: true
+           filename: output.pdf
+
     """
 
+    def _create_plot(self):
+        """Actual drawing of datasets."""
+        super()._create_plot()
+        if self.parameters['g-axis'] \
+                and self.datasets[0].data.axes[0].unit == 'mT':
+            self._create_g_axis(
+                self.datasets[0].metadata.bridge.mw_frequency.value)
 
-class MultiPlotter1DStacked(aspecd.plotting.MultiPlotter1DStacked):
+
+class MultiPlotter1DStacked(aspecd.plotting.MultiPlotter1DStacked,
+                            PlotterExtensions):
     """Stacked 1D plots of multiple datasets.
 
     Convenience class taking care of 1D plots of multiple datasets.
@@ -551,6 +572,9 @@ class MultiPlotter1DStacked(aspecd.plotting.MultiPlotter1DStacked):
     ASpecD documentation of the :class:`aspecd.plotting.MultiPlotter1DStacked`
     class for details.
 
+    Furthermore, the class inherits all functionality from
+    :class:`PlotterExtensions`. See there for additional details.
+
     Examples
     --------
     For convenience, a series of examples in recipe style (for details of
@@ -606,50 +630,24 @@ class MultiPlotter1DStacked(aspecd.plotting.MultiPlotter1DStacked):
            parameters:
              show_zero_lines: True
 
-    """
+    In case you would like to have a *g* axis plotted as a second *x* axis on
+    top (note that this only makes sense in case of a calibrated magnetic
+    field axis):
 
+    .. code-block:: yaml
 
-class CompositePlotter(aspecd.plotting.CompositePlotter):
-    """Base class for plots consisting of multiple axes.
-
-    The underlying idea of composite plotters is to use a dedicated
-    existing plotter for each axis and assign this plotter to the list of
-    plotters of the CompositePlotter object. Thus the actual plotting task
-    is left to the individual plotter and the CompositePlotter only takes
-    care of the specifics of plots consisting of more than one axis.
-
-    As the class is fully inherited from ASpecD for simple usage, see the
-    ASpecD documentation of the :class:`aspecd.plotting.CompositePlotter`
-    class for details.
+       - kind: multiplot
+         type: MultiPlotter1DStacked
+         properties:
+           parameters:
+             g-axis: true
+           filename: output.pdf
 
     """
 
-
-class SingleCompositePlotter(aspecd.plotting.SingleCompositePlotter):
-    """Composite plotter for single datasets.
-
-    This composite plotter is used for different representations of one and the
-    same dataset in multiple axes contained in one figure. In this respect,
-    it works like all the other ordinary single plotters derived from
-    :class:`SinglePlotter`, *i.e.* it usually gets called by using the dataset's
-    :meth:`aspecd.dataset.Dataset.plot` method.
-
-    As the class is fully inherited from ASpecD for simple usage, see the
-    ASpecD documentation of the :class:`aspecd.plotting.SingleCompositePlotter`
-    class for details.
-
-    """
-
-
-class Saver(aspecd.plotting.Saver):
-    """Base class for saving plots.
-
-    For basic saving of plots, no subclassing is necessary, as the
-    :meth:`save` method uses :meth:`matplotlib.figure.Figure.savefig` and
-    can cope with all possible parameters via the :attr:`parameters` property.
-
-    As the class is fully inherited from ASpecD for simple usage, see the
-    ASpecD documentation of the :class:`aspecd.plotting.Saver` class for
-    details.
-
-    """
+    def _create_plot(self):
+        super()._create_plot()
+        if self.parameters['g-axis'] \
+                and self.datasets[0].data.axes[0].unit == 'mT':
+            self._create_g_axis(
+                self.datasets[0].metadata.bridge.mw_frequency.value)
